from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.conf import settings

class UserSAMLIdentifier(models.Model):
    user    = models.ForeignKey(
                 verbose_name=_('user'),
                 to=settings.AUTH_USER_MODEL,
                 related_name='saml_identifiers')
    issuer  = models.TextField(
                 verbose_name=_('Issuer'))
    name_id = models.TextField(
                 verbose_name=_('SAML identifier'))
    created = models.DateTimeField(
                 verbose_name=_('created'),
                 auto_now_add=True)

    class Meta:
        verbose_name = _('user SAML identifier')
        verbose_name_plural = _('users SAML identifiers')
        unique_together = (('issuer', 'name_id'),)



from django.conf import global_settings

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': 'mellon.sqlite3',
    }
}
DEBUG = True
SECRET_KEY='xx'
INSTALLED_APPS = ('mellon', 'django.contrib.auth', 'django.contrib.contenttypes', 'django.contrib.sessions')
MIDDLEWARE_CLASSES = global_settings.MIDDLEWARE_CLASSES



import pytest

from django.conf import settings
from django.contrib import auth

from mellon.adapters import DefaultAdapter

pytestmark = pytest.mark.django_db

idp = {}
saml_attributes = {
    'name_id_content': 'x'*32,
    'issuer': 'https://idp.example.net/saml/metadata',
    'username': ['foobar'],
    'email': ['test@example.net'],
    'first_name': ['Foo'],
    'last_name': ['Bar'],
    'is_superuser': ['true'],
}

def test_format_username(settings):
    adapter = DefaultAdapter()
    assert adapter.format_username(idp, {}) == None
    assert adapter.format_username(idp, saml_attributes) == ('x'*32 + '@saml')[:30]
    settings.MELLON_USERNAME_TEMPLATE = '{attributes[name_id_content]}'
    assert adapter.format_username(idp, saml_attributes) == ('x'*32)[:30]
    settings.MELLON_USERNAME_TEMPLATE = '{attributes[username][0]}'
    assert adapter.format_username(idp, saml_attributes) == 'foobar'

def test_lookup_user(settings):
    User = auth.get_user_model()
    adapter = DefaultAdapter()
    user = adapter.lookup_user(idp, saml_attributes)
    assert user is not None

    user2 = adapter.lookup_user(idp, saml_attributes)
    assert user.id == user2.id

    User.objects.all().delete()
    assert User.objects.count() == 0

    settings.MELLON_PROVISION = False
    user = adapter.lookup_user(idp, saml_attributes)
    assert user is None
    assert User.objects.count() == 0

def test_provision(settings):
    User = auth.get_user_model()
    adapter = DefaultAdapter()
    settings.MELLON_ATTRIBUTE_MAPPING = {
        'email': '{attributes[email][0]}',
        'first_name': '{attributes[first_name][0]}',
        'last_name': '{attributes[last_name][0]}',
    }
    user = User(username='xx')
    user.save()
    adapter.provision(user, idp, saml_attributes)
    assert user.first_name == 'Foo'
    assert user.last_name == 'Bar'
    assert user.email == 'test@example.net'
    assert user.is_superuser == False
    User.objects.all().delete()

    settings.MELLON_SUPERUSER_MAPPING = {
        'is_superuser': 'true',
    }
    user = User(username='xx')
    user.save()
    adapter.provision(user, idp, saml_attributes)
    assert user.is_superuser == True
    User.objects.all().delete()

    local_saml_attributes = saml_attributes.copy()
    del local_saml_attributes['email']
    user = User(username='xx')
    user.save()
    adapter.provision(user, idp, local_saml_attributes)
    assert not user.email
    User.objects.all().delete()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='UserSAMLIdentifier',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('issuer', models.TextField(verbose_name='Issuer')),
                ('name_id', models.TextField(verbose_name='SAML identifier')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='created')),
                ('user', models.ForeignKey(related_name='saml_identifiers', verbose_name='user', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name': 'user SAML identifier',
                'verbose_name_plural': 'users SAML identifiers',
            },
            bases=(models.Model,),
        ),
        migrations.AlterUniqueTogether(
            name='usersamlidentifier',
            unique_together=set([('issuer', 'name_id')]),
        ),
    ]
